SELECT * FROM sys.procedures ORDER BY create_date DESC

--procedure stats
SELECT	OBJECT_NAME([object_id]) AS [procedure_name],
		cached_time,
		last_execution_time,
		execution_count,
		CAST(ps.total_worker_time / CAST(execution_count AS float) / 1000.0 AS decimal(19,2)) AS avg_worker_ms,
		CAST(total_worker_time/1000000.0 AS decimal(19,2)) AS total_worker_sec,
		CAST(CASE WHEN DATEDIFF(minute,cached_time,last_execution_time) < 1 THEN 0 ELSE execution_count / CAST(DATEDIFF(minute,cached_time,last_execution_time) AS decimal(19,2)) END AS decimal(19,2)) AS execs_per_min,
		total_logical_reads,
		CAST(total_logical_reads / CAST(execution_count AS float) AS decimal(19,2)) AS reads_per_exec,
		total_logical_writes,
		CAST(total_elapsed_time /1000000.0 AS decimal(19,2)) AS total_elapsed_sec,
		(SELECT text FROM sys.dm_exec_sql_text([sql_handle])) AS sql_text,
		(SELECT query_plan FROM sys.dm_exec_query_plan(plan_handle)) AS query_plan
FROM sys.dm_exec_procedure_stats ps
WHERE database_id = DB_ID()
--ORDER BY total_worker_time DESC
ORDER BY total_worker_time/CAST(execution_count AS float) DESC;

/*
CREATE INDEX [ix_Product_ProductCategoryID] ON Product (ProductCategoryID) 
INCLUDE (ProductName,ProductPrice) WITH DROP_EXISTING
*/
-- query for query stats
SELECT TOP 100
		creation_time, 
		last_execution_time, 
		execution_count, 
		CAST(total_worker_time/1000000.0 AS decimal(19,2)) AS total_worker_sec,
		CAST(ps.total_worker_time / CAST(execution_count AS float) / 1000.0 AS decimal(19,2)) AS avg_worker_ms,
		CAST(CASE WHEN DATEDIFF(minute,creation_time,last_execution_time) < 1 THEN 0 ELSE execution_count / CAST(DATEDIFF(minute,creation_time,last_execution_time) AS decimal(19,2)) END AS decimal(19,2)) AS execs_per_min,
		total_logical_reads,
		total_logical_writes,
		CAST(total_elapsed_time /1000000.0 AS decimal(19,2)) AS total_elapsed_sec,
		SUBSTRING(st.text, (qs.statement_start_offset/2) + 1,
			((CASE statement_end_offset 
				WHEN -1 THEN DATALENGTH(st.text)
				ELSE qs.statement_end_offset END 
					- qs.statement_start_offset)/2) + 1) as statement_text
FROM sys.dm_exec_query_stats as qs
	CROSS APPLY sys.dm_exec_sql_text(qs.sql_handle) as st
WHERE st.dbid = DB_ID()
ORDER BY total_worker_time DESC
ORDER BY total_worker_time/CAST(execution_count AS float) DESC;

-- query for index usage stats
SELECT	sd.crdate as server_start_time,
		OBJECT_NAME(si.[object_id]) AS table_name,
		si.name AS index_name,
		si.type_desc, 
		si.is_unique,
		RTRIM(SUBSTRING(
		(	SELECT ',' + sc.name + ' (' + sy.name + ')' AS [text()]
			FROM sys.index_columns ic
				JOIN sys.columns sc ON ic.object_id = sc.object_id
									AND ic.column_id = sc.column_id
				JOIN sys.types sy ON sc.user_type_id = sy.user_type_id
			WHERE sc.object_id = si.object_id
				AND ic.index_id = si.index_id
				AND ic.is_included_column = 0
			ORDER BY ic.key_ordinal
			FOR XML PATH('')),2,999)) AS column_list,
		RTRIM(SUBSTRING(
		(	SELECT ',' + sc.name + ' (' + sy.name + ')' AS [text()]
			FROM sys.index_columns ic
				JOIN sys.columns sc ON ic.object_id = sc.object_id
									AND ic.column_id = sc.column_id
				JOIN sys.types sy ON sc.user_type_id = sy.user_type_id
			WHERE sc.object_id = si.object_id
				AND ic.index_id = si.index_id
				AND ic.is_included_column = 1
			ORDER BY ic.key_ordinal
			FOR XML PATH('')),2,999)) AS included_columns,
		si.[object_id],
		si.index_id,
		user_seeks,
		user_scans,
		user_lookups,
		user_updates
FROM (SELECT * FROM sys.dm_db_index_usage_stats WHERE database_id = db_id())  iu
	RIGHT JOIN sys.indexes si
		JOIN sys.tables st ON si.[object_id] = st.[object_id] 
		ON iu.[object_id] = si.[object_id] AND iu.index_id = si.index_id
	CROSS JOIN master.sys.sysdatabases sd
WHERE sd.name = 'tempdb'
ORDER BY si.[object_id]

-- query for missing index stats
SELECT	sd.crdate as server_start_time,
		mid.[object_id],
		mid.[statement],
		mid.equality_columns,
		mid.inequality_columns,
		mid.included_columns,
		mis.unique_compiles,
		mis.user_seeks,
		mis.user_scans,
		mis.avg_total_user_cost,
		mis.avg_user_impact
FROM sys.dm_db_missing_index_details mid
	JOIN sys.dm_db_missing_index_groups mig ON mid.index_handle = mig.index_handle
	JOIN sys.dm_db_missing_index_group_stats mis ON mig.index_group_handle = mis.group_handle
	CROSS JOIN master.sys.sysdatabases sd
WHERE mid.database_id = DB_ID()
	AND sd.name = 'tempdb'

